"""
Created on 26 mars 2018

@author: Stephane Tonneau

This module requires the installation of the python kafka library with
    pip install kafka-python

"""
import sys
from pidclient.base_utils import LoggingMethod, Tracer
try:
    import gssapi
except ImportError:
    pass
# The next line is required to avoid the import of snappy as VITO
# has also a snappy package from ESA (with different functionalities).
# The side effect of the ESA package is that it induces a 'too many file open'
# issue for kafka producer.
sys.modules['snappy'] = None
try:
    from kafka import KafkaProducer
except ImportError:
    # the unavailibility of the package shall be treated later
    # when trying to use a producer
    pass

# Topic writing in kafka need a list of bytes
# in python 3.x, the function is bytes(source,'utf-8')
# while bytes(source) is required in python 2.x
#
try:
    THE_BYTES = bytes('xxx')
except:

    def to_topic(source):
        """ return the string as a byte encoded list of characters for python 3.x"""
        return bytes(source, 'utf-8')

else:

    def to_topic(source):
        """ return the string as a byte encoded list of characters for python 2.x"""
        return bytes(source)

NODENAME = "kafka"


class KafkaLogging(LoggingMethod):
    """Class that encapsulates the storing of objects in a kafka topic
    for logging purposes.
    """

    def __init__(self, **parameters):
        """Initialisator of the class

        :param configuration: the general logging configuration
        :type configuration: a python configuration object
        """
        super(KafkaLogging, self).__init__(**parameters)
        self.es_index = self.get_config(NODENAME, "es_index")
        self.es_type = self.get_config(NODENAME, "es_type")

        kafka_brokers = self.get_config(NODENAME, "brokers")
        if kafka_brokers is None:
            self.kafka_brokers = None
            Tracer.error("KafkaLogging",
                         "Missing brokers configuration value")
        else:
            self.kafka_brokers = kafka_brokers.replace(" ", "").split(",")
        self.kafka_topic = self.get_config(NODENAME, "topic")
        if self.kafka_topic is None:
            Tracer.error("KafkaLogging",
                         "Missing topic configuration value")
        self.secure = self.get_config(NODENAME, "secure")
        try:
            if self.secure is None:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, acks='all')
            else:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_brokers, security_protocol='SASL_PLAINTEXT',
                    sasl_mechanism='GSSAPI', acks='all')
            if self.producer is None:
                Tracer.error("KafkaLogging",
                             "Producer NOT CREATED !!!")
        except (Exception) as error:
            Tracer.error("KafkaLogging",
                         "unable to create a producer " + str(error))
            Tracer.error("KafkaLogging", "kafka brokers : " + str(self.kafka_brokers) +
                         " - topic " + str(self.kafka_topic) + " - mode " + str(self.secure))
            self.producer = None

    def close(self):
        """Close the logging instance"""
        if self.producer is not None:
            try:
                self.producer.flush()
                self.producer.close()
            except:
                Tracer.error("KafkaLogging",
                             "unable to close kafka system")
                Tracer.error("KafkaLogging", "kafka brokers : " + str(self.kafka_brokers) +
                             " - topic " + str(self.kafka_topic) + " - mode " + self.secure)
            self.producer = None

    def process_log(self, entry, meta_information):
        """Method to insert a json representation of both an object
        and its metadata in a kafka topic

        :param entry: the standard information to log
        :type entry: PidEntry
        :param meta_information: meta information about the information to log
        :type meta_information: Metainformation
        """
        res = None
        if self.producer is not None:
            try:
                entry.id = meta_information.uid
                if self.es_index is not None:
                    entry.es_index = self.es_index
                if self.es_type is not None:
                    entry.es_type = self.es_type
                future = self.producer.send(
                    self.kafka_topic, to_topic(str(entry.to_json())))
                res = future.get(timeout=60)
            except (Exception) as error:
                Tracer.error("KafkaLogging",
                             "unable to write the logging entry (" + str(error) + ") : " +
                             str(entry.to_json()))
                Tracer.error("KafkaLogging",
                             "kafka brokers : " + str(self.kafka_brokers) + " - topic " +
                             str(self.kafka_topic) + " - mode " + str(self.secure))
        return res
